////////////////////////////
// ARRAY ADT (templated)
//
// Author: Jonathan Lavering
//
////////////////////////////

#ifndef ARRAY_H
#define ARRAY_H

#include "OutOfBounds.h"

template <typename T>
class Array
{ 	
public:
		Array(); // default constructor
		~Array(); // default destructor
		Array( int length, int base = 0 ); // constructor
		Array( const Array< T > & copy ); // copy constructor
		Array< T > & operator = ( const Array< T > & array ); // operator equals
		T & operator [] ( int index ); // index operator
		const T * getData() const;
		int getBase() const;
		int getLength() const;
		void setBase( int newBase );
		void setLength( int newLength );
private:
		T * m_data;
		int m_base;
		int m_length;
};

template <typename T>
Array< T >::Array()
{
	m_data = 0;
	m_base = 0;
	m_length = 0;
}

template <typename T>
Array< T >::~Array()
{
	delete [] m_data;
	m_base = 0;
	m_length = 0;
}

template <typename T>
Array< T >::Array( int length, int base )
{
	//throw exception for neg length
	if( length < 0 )
		throw OutOfBounds("ERROR: cannot have negative length.");

	m_length = length;
	m_base = base;
	m_data = new T[ length ];
}

template <typename T>
Array< T >::Array( const Array< T > & copy )
{
	*this = copy;
}

template <typename T>
Array< T > & Array< T >::operator = ( const Array< T > & array )
{
	if( this != &array )
	{
		// delete current array if it exists
		delete [] m_data;

		m_length = array.getLength();
		m_base = array.getBase();
		m_data = new T[ m_length ];

		for( int i = 0; i < m_length; i++ )
		{
			m_data[i] = array.m_data[i];
		}
	}
	return *this;
}

template <typename T>
T & Array< T >::operator [] ( int index )
{
	// index range: [ base -> ( base + length ) ]
	// so, out of bounds occurs when the index is less than the base and
	// when the index minus the base is greater than or equal to the length
	if( ( index < m_base ) || ( ( index - m_base ) >= m_length ) )
	{
		throw OutOfBounds( "ERROR: Index out of bounds" );
	}
	return m_data[ index - m_base ];
}

template <typename T>
const T * Array< T >::getData() const
{
	return m_data;
}

template <typename T>
int Array< T >::getBase() const
{
	return m_base;
}

template <typename T>
int Array< T >::getLength() const
{
	return m_length;
}

template <typename T>
void Array< T >::setBase( int newBase )
{
	m_base = newBase;
}

template <typename T>
void Array< T >::setLength( int newLength )
{
	// check for negative length--throw exception if so
	if( newLength < 0 )
		throw OutOfBounds("ERROR: cannot have negative length");

	// if the lengths are equal, no change
	if( m_length == newLength )
		return;

	// if the new length is equal to 0, just clear the array
	if( newLength == 0 )
	{
		delete [] m_data;
		m_data = 0;
		m_length = 0;
		return;
	}

	if( m_data ) // if the array is not empty
	{
		T * temp = new T[newLength]; // make temp array of new length

		// if array is decreasing in size
		if( m_length > newLength )
		{
			// copy up to the amount length shortening too
			for( int i = 0; i < newLength; i++ )
				temp[i] = m_data[i];
		}
		else // if array is increasing in size
		{
			// copy up to the origonal length
			for( int i = 0; i < m_length; i++ )
				temp[i] = m_data[i];
		}

		delete [] m_data;
		m_data = temp;
	}
	else // if the array is empty
	{
		m_data = new T[newLength];
	}
	m_length = newLength;
}

#endif